using developwithpassion.bdd;
using developwithpassion.bdd.contexts;
using developwithpassion.bdd.mbunit.standard.observations;

namespace Domain.Specs
{
    public class CommandParserSpecs
    {
        public abstract class concern : observations_for_a_sut_without_a_contract<CommandParser> {
            context c = () =>
            {
                saleEventListener = the_dependency<ISaleEventListener>();
            };
            static protected ISaleEventListener saleEventListener;
            static protected string end_sale_message;
        }

        public class when_interpreting_an_new_sale : concern
        {
            context c = () =>
                end_sale_message = "Command:NewSale";

            because b = () =>
                sut.Parse(end_sale_message);

            it should_notify_listener_of_a_new_sale = () =>
                saleEventListener.received(x => x.NewSaleInitiated());
        }

        public class when_interpreting_an_end_sale : concern
        {
            context c = () =>
                end_sale_message = "Command:EndSale";

            because b = () =>
                sut.Parse(end_sale_message);

            it should_notify_listner_to_complete_the_sale = () =>
                saleEventListener.received(x => x.SaleCompleted());
        }

        public class when_interpreting_an_item_being_entered  : concern
        {
            context c = () =>
            {
                end_sale_message = "Input: Barcode=100008888559, Quantity=1";
                expected_itemid = new ItemId("100008888559");
                expected_quantity = new Quantity(1);
            };

            because b = () =>
                sut.Parse(end_sale_message);

            it should_notify_listener_of_item_and_quantity_entered = () =>
                saleEventListener.received(x => x.ItemEntered(expected_itemid,expected_quantity));

            static ItemId expected_itemid;
            static Quantity expected_quantity;
        }
    }
}