using developwithpassion.bdd;
using developwithpassion.bdd.contexts;
using developwithpassion.bdd.mbunit.standard.observations;
using Rhino.Mocks;

namespace Domain.Specs
{
    public class RegisterSpecs
    {
        public class concern : observations_for_a_sut_without_a_contract<Register>
        {
            context c = () =>
            {
                receiptReceiver = the_dependency<IReceiptReceiver>();
                product_catalog = the_dependency<IProductCatalog>();
            };

            static protected IReceiptReceiver receiptReceiver;
            static protected IProductCatalog product_catalog;
        }

        public class when_completing_a_sale_that_was_started_but_has_no_items_added : concern
        {
            after_the_sut_has_been_created a = () =>
                                               sut.NewSaleInitiated();

            because b = () =>
                        sut.SaleCompleted();

            it should_calculate_a_sales_receipt_of_zero = () =>
                                                          receiptReceiver.received(x => x.ReceiveTotalDue(zero_dollars));

            private static readonly Money zero_dollars = new Money(0m);
        }

        public class when_attempting_to_complete_a_sale_that_was_not_started : concern
        {
            because b = () =>
                        sut.SaleCompleted();

            it should_not_create_a_receipt = () =>
                                             receiptReceiver.never_received(x => x.ReceiveTotalDue(Arg<Money>.Is.Anything));
        }

        public class when_completing_a_sale_that_has_multiple_items_added_to_it : concern
        {
            context c = () =>
            {
                item_id_1 = new ItemId("000000001");
                item_id_2 = new ItemId("000000002");
                description_for_item_with_id1 = new ProductDescription("description 1", when_completing_a_sale_that_has_multiple_items_added_to_it.price_of_item_1);
                price_of_item_1 = new Money(3.00m);
                price_of_item_2 = new Money(7.00m);
                description_for_item_with_id2 = new ProductDescription("description 2", when_completing_a_sale_that_has_multiple_items_added_to_it.price_of_item_2);
                single_item = new Quantity(1);

                product_catalog.Stub(x => x.ProductDescriptionFor(item_id_1)).Return(description_for_item_with_id1);
                product_catalog.Stub(x => x.ProductDescriptionFor(item_id_2)).Return(description_for_item_with_id2);
            };

            after_the_sut_has_been_created a = () =>
            {
                sut.NewSaleInitiated();
                sut.ItemEntered(item_id_1, single_item);
                sut.ItemEntered(item_id_2, single_item);
            };

            because b = () =>
                        sut.SaleCompleted();

            it should_prepare_a_sales_receipt_with_the_total_cost_of_the_items_added = () =>
                           receiptReceiver.received(x => x.ReceiveTotalDue(price_of_item_1 + price_of_item_2));



            static ItemId item_id_1;
            static ItemId item_id_2;
            static ProductDescription description_for_item_with_id1;
            static ProductDescription description_for_item_with_id2;
            static Quantity single_item;
            private static Money price_of_item_1;
            private static Money price_of_item_2;
        }
    }
}