using NMock2;
using NUnit.Framework;

namespace Domain.Tests
{
    [TestFixture]
    public class CommandParserTests
    {
        private Mockery mockery;
        private CommandParser commandParser;
        private ISaleEventListener saleEventListener;

        [SetUp]
        public void BeforeTest()
        {
            mockery = new Mockery();
            saleEventListener = mockery.NewMock<ISaleEventListener>();
            commandParser = new CommandParser(saleEventListener);
        }

        [TearDown]
        public void AfterTest()
        {
            mockery.VerifyAllExpectationsHaveBeenMet();

        }

        [Test]
        public void NotifiesListenerOfNewSaleEvent()
        {
            const string newSaleCommand = "Command:NewSale";

            Expect.Once.On(saleEventListener).Method("NewSaleInitiated");

            commandParser.Parse(newSaleCommand);
        }


        [Test]
        public void NotifiesListenerOfSaleCompletedEvent()
        {
            const string endSaleCommand = "Command:EndSale";

            Expect.Once.On(saleEventListener).Method("SaleCompleted");

            commandParser.Parse(endSaleCommand);
        }

        
        [Test]
        public void NotifiesListenerOfItemAndQuantityEntered()
        {
            const string commandWithBarcodeAndSingleQuantity = "Input: Barcode=100008888559, Quantity=1";
            var expectedItemId = new ItemId("100008888559");
            var singleQuantity = new Quantity(1);

            Expect.Once.On(saleEventListener).Method("ItemEntered").With(expectedItemId, singleQuantity);

            commandParser.Parse(commandWithBarcodeAndSingleQuantity);
        }

    }
}