using NMock2;
using NUnit.Framework;

namespace Domain.Tests
{
    [TestFixture]
    public class RegisterTests
    {
        private Mockery mockery;
        private IReceiptReceiver receiptReceiver;
        private Register register;
        private readonly ItemId itemId1 = new ItemId("000000001");
        private readonly ItemId itemId2 = new ItemId("000000002");
        private static readonly Money priceOfItem2 = new Money(7.00m);
        private static readonly Money priceOfItem1 = new Money(3.00m);

        private readonly ProductDescription descriptionForItemWithId1 = new ProductDescription("description 1", priceOfItem1);

        private readonly ProductDescription descriptionForItemWithId2 = new ProductDescription("description 2", priceOfItem2);
        private readonly Quantity singleItem = new Quantity(1);
        private IProductCatalog productCatalog;
        

        [SetUp]
        public void BeforeTest()
        {
            mockery = new Mockery();
            receiptReceiver = mockery.NewMock<IReceiptReceiver>();
            productCatalog = mockery.NewMock<IProductCatalog>();
            register = new Register(receiptReceiver, productCatalog);

            Stub.On(productCatalog).Method("ProductDescriptionFor").With(itemId1)

                                   .Will(Return.Value(descriptionForItemWithId1));

            Stub.On(productCatalog).Method("ProductDescriptionFor").With(itemId2)

                                  .Will(Return.Value(descriptionForItemWithId2));

        }
        
        [TearDown]
        public void AfterTest()
        {
            mockery.VerifyAllExpectationsHaveBeenMet();
        }

        [Test]
        public void ReceiptTotalForASaleWithNoItemsShouldBeZero()
        {
           
            register.NewSaleInitiated();

            var zeroDollars = new Money(0m);
            Expect.Once.On(receiptReceiver).Method("ReceiveTotalDue").With(zeroDollars);

            register.SaleCompleted();
        }


        [Test]
        public void ShouldNotSendRecieptWhenThereIsNoSale()
        {
            Expect.Never.On(receiptReceiver);
            register.SaleCompleted();
        }

        [Test]
        public void ShouldCalculateRecieptForSaleWithMultipleItemsOfSingleQuantity()
        {
            register.NewSaleInitiated();

            register.ItemEntered(itemId1, singleItem);

            register.ItemEntered(itemId2, singleItem);


            Expect.Once.On(receiptReceiver).Method("ReceiveTotalDue").With(priceOfItem1 + priceOfItem2);


            register.SaleCompleted();
        }


    }
}