using System.Collections.Generic;

namespace Domain
{
    public class CommandParser
    {
        private const string EndSaleCommand = "EndSale";
        private readonly ISaleEventListener saleEventListener;
        private const string Input = "Input";
        private const string StartSaleCommand = "NewSale";

        public CommandParser(ISaleEventListener saleEventListner)
        {
            this.saleEventListener = saleEventListner;
        }

        public void Parse(string messageFromDevice)
        {
            var command = messageFromDevice.Split(':');
            var commandType = command[0].Trim();
            var commandBody = command[1].Trim();
            if(Input.Equals(commandType))
            {
                ProcessInputCommand(commandBody);
            }
            else
            {
                ProcessCommand(commandBody);
            }
        }

        private void ProcessCommand(string commandBody)
        {
            if (EndSaleCommand.Equals(commandBody))
                saleEventListener.SaleCompleted();
            else if (StartSaleCommand.Equals(commandBody)) 
                saleEventListener.NewSaleInitiated();
        }

        private void ProcessInputCommand(string commandBody)
        {
            var arguments = new Dictionary<string, string>();
            var commandArgs = commandBody.Split(',');
            foreach(var argument in commandArgs)
            {
                var argNameValues = argument.Split('=');
                arguments.Add(argNameValues[0].Trim(), argNameValues[1].Trim());
            }

            string  barcode = arguments["Barcode"];
            var quantity = int.Parse(arguments["Quantity"]);

            saleEventListener.ItemEntered(new ItemId(barcode), new Quantity(quantity));
        }
    }
}