namespace Domain
{
    public class Register : ISaleEventListener
    {
        private readonly IReceiptReceiver receiptReceiver;
        private readonly IProductCatalog productCatalog;
        private static readonly Sale nullSale = new NullSale();
        private Sale currentSale;


        
        public Register(IReceiptReceiver receiver, IProductCatalog productCatalog)
        {
            this.receiptReceiver = receiver;
            this.productCatalog = productCatalog;
            currentSale = nullSale;
        }

        public void SaleCompleted()
        {
            currentSale.SendReceiptTo(receiptReceiver);
        }

        public void NewSaleInitiated()
        {
            currentSale = new Sale();
        }

        public void ItemEntered(ItemId itemId, Quantity quantity)
        {
            var productDescription = productCatalog.ProductDescriptionFor(itemId);
            currentSale.PurchaseItemWith(productDescription);
        }
    }
}